{ ####################################################################################### }
{ ##                                                                                   ## }
{ ## ADS1262_Unit                                                                      ## }
{ ##                                                                                   ## }
{ ## Library for ADS1262                                                               ## }
{ ## Based on:                                                                         ## }
{ ## - Timm Thaler: http://wiki.lazarus.freepascal.org/index.php?title=<br>            ## }
{ ##        Raspberry_Pi_-_SPI/de&oldid=113577,                                        ## }
{ ## - Professor Plate: http://www.netzmafia.de/skripten/hardware/RasPi/RasPi_SPI.html,## }
{ ## - Blake Felt library: https://github.com/Molorius/ADS126X and                     ## }
{ ## - ProtoCentral library: https://github.com/Protocentral/ProtoCentral_ads1262.     ## }
{ ## - pascalio library: https://github.com/SAmeis/pascalio                            ## }
{ ##                                                                                   ## }
{ ## Copyright (C) 2018-2019  : Dr. Jürgen Abel                                        ## }
{ ## Email                    : juergen@mve.info                                       ## }
{ ## Internet                 : www.seismometer.info                                   ## }
{ ##                                                                                   ## }
{ ## This program is free software: you can redistribute it and/or modify              ## }
{ ## it under the terms of the GNU Lesser General Public License as published by       ## }
{ ## the Free Software Foundation, either version 3 of the License, or                 ## }
{ ## (at your option) any later version with the following modification:               ## }
{ ##                                                                                   ## }
{ ## As a special exception, the copyright holders of this library give you            ## }
{ ## permission to link this library with independent modules to produce an            ## }
{ ## executable, regardless of the license terms of these independent modules, and     ## }
{ ## to copy and distribute the resulting executable under terms of your choice,       ## }
{ ## provided that you also meet, for each linked independent module, the terms        ## }
{ ## and conditions of the license of that module. An independent module is a          ## }
{ ## module which is not derived from or based on this library. If you modify          ## }
{ ## this library, you may extend this exception to your version of the library,       ## }
{ ## but you are not obligated to do so. If you do not wish to do so, delete this      ## }
{ ## exception statement from your version.                                            ## }
{ ##                                                                                   ## }
{ ## This program is distributed in the hope that it will be useful,                   ## }
{ ## but WITHOUT ANY WARRANTY; without even the implied warranty of                    ## }
{ ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                     ## }
{ ## GNU General Public License for more details.                                      ## }
{ ##                                                                                   ## }
{ ## You should have received a copy of the GNU Lesser General Public License          ## }
{ ## COPYING.LGPL.txt along with this program.                                         ## }
{ ## If not, see <https://www.gnu.org/licenses/>.                                      ## }
{ ##                                                                                   ## }
{ ####################################################################################### }

Unit ADS1262_Unit;

{$mode objfpc}{$H+}

{$INCLUDE project_defines.inc}

Interface

Uses
  CThreads,
  Classes;

Const
  ADS1262_REGISTER_N = $15;

Type
  { ####################################################################################### }
  { ## Number types                                                                      ## }
  { ####################################################################################### }
  Int_8     = Int8;
  P_Int_8   = ^Int_8;
  U_Int_8   = UInt8;
  P_U_Int_8 = ^U_Int_8;
  Char      = Int_8;
  P_Char    = ^Char;
  U_Char    = U_Int_8;
  P_U_Char  = ^U_Char;

  Int_16     = Int16;
  P_Int_16   = ^Int_16;
  U_Int_16   = UInt16;
  P_U_Int_16 = ^U_Int_16;

  Int_32     = Int32;
  P_Int_32   = ^Int_32;
  U_Int_32   = UInt32;
  P_U_Int_32 = ^U_Int_32;

  Int_64     = Int64;
  P_Int_64   = ^Int_64;
  U_Int_64   = UInt64;
  P_U_Int_64 = ^U_Int_64;

  Bool   = Bytebool;
  P_Bool = ^Bool;

  T_GPIO_Direction            = (GPIO_DIRECTION_IN, GPIO_DIRECTION_OUT);
  T_GPIO_Interrupt_Mode_Types = (GPIO_INTERRUPT_RISING, GPIO_INTERRUPT_FALLING);
  T_GPIO_Interrupt_Mode       = Set Of T_GPIO_Interrupt_Mode_Types;

  T_Buffer_A = Array Of U_Int_8;

  T_SPI_Transfer_R = Record
      tx_buf :        U_Int_64;
      rx_buf :        U_Int_64;
      len :           U_Int_32;
      speed_hz :      U_Int_32;
      delay_usecs :   U_Int_16;
      bits_per_word : U_Int_8;
      cs_change :     U_Int_8;
      pad :           U_Int_32;
  End; { T_SPI_Transfer_R }

  T_Register_Block = Array [0 .. ADS1262_REGISTER_N - 1] Of U_Int_8;

Const
  { ####################################################################################### }
  { ## GPIO data                                                                         ## }
  { ####################################################################################### }
  GPIO_LINUX_BASE_DIR             = '/sys/class/gpio/';
  GPIO_LINUX_GPIOPIN_DIR          = GPIO_LINUX_BASE_DIR + 'gpio%d/';
  GPIO_LINUX_EXPORT_FILE          = GPIO_LINUX_BASE_DIR + 'export';
  GPIO_LINUX_UNEXPORT_FILE        = GPIO_LINUX_BASE_DIR + 'unexport';
  GPIO_DRDY_PIN                   = 25;
  GPIO_WAIT_TIME_MS_EXPORT        = 100;
  GPIO_WAIT_TIME_MS_COMMAND       = 10;
  GPIO_TIMEOUT_INTERRUPTMODE_MS   = 100;
  GPIO_TIMEOUT_INTERRUPT_INFINITE = -1;
  GPIO_INTERRUPTMODE_MS           = 100;

  { ####################################################################################### }
  { ## SPI general data                                                                  ## }
  { ####################################################################################### }
  SPI_IOC_DEVICE               = '/dev/spidev0.0';
  SPI_IOC_WAIT_TIME_MS_OPEN    = 100;
  SPI_IOC_WAIT_TIME_MS_COMMAND = 10;

  { ####################################################################################### }
  { ## SPI request commands                                                              ## }
  { ####################################################################################### }
  SPI_IOC_MODE_READ  = $80;
  SPI_IOC_MODE_WRITE = $40;
  SPI_IOC_MODE_CTRL  = $6B;

  SPI_IOC_RD_MODE : U_Int_32          = (SPI_IOC_MODE_READ shl $18) or ($01 shl $10) or (SPI_IOC_MODE_CTRL shl $08) or ($01);       { $80016B01; }
  SPI_IOC_WR_MODE : U_Int_32          = (SPI_IOC_MODE_WRITE shl $18) or ($01 shl $10) or (SPI_IOC_MODE_CTRL shl $08) or ($01);      { $40016B01; }
  SPI_IOC_RD_BITS_PER_WORD : U_Int_32 = (SPI_IOC_MODE_READ shl $18) or ($01 shl $10) or (SPI_IOC_MODE_CTRL shl $08) or ($03);       { $80016B03; }
  SPI_IOC_WR_BITS_PER_WORD : U_Int_32 = (SPI_IOC_MODE_WRITE shl $18) or ($01 shl $10) or (SPI_IOC_MODE_CTRL shl $08) or ($03);      { $40016B03; }
  SPI_IOC_RD_LSB_FIRST : U_Int_32     = (SPI_IOC_MODE_READ shl $18) or ($01 shl $10) or (SPI_IOC_MODE_CTRL shl $08) or ($02);       { $80016B02; }
  SPI_IOC_WR_LSB_FIRST : U_Int_32     = (SPI_IOC_MODE_WRITE shl $18) or ($01 shl $10) or (SPI_IOC_MODE_CTRL shl $08) or ($02);      { $40016B02; }
  SPI_IOC_RD_MAX_SPEED_HZ : U_Int_32  = (SPI_IOC_MODE_READ shl $18) or ($04 shl $10) or (SPI_IOC_MODE_CTRL shl $08) or ($04);       { $80046B04; }
  SPI_IOC_WR_MAX_SPEED_HZ : U_Int_32  = (SPI_IOC_MODE_WRITE shl $18) or ($04 shl $10) or (SPI_IOC_MODE_CTRL shl $08) or ($04);      { $40046B04; }
  SPI_IOC_WR_TRANSFER : U_Int_32      = (SPI_IOC_MODE_WRITE shl $18) or ($20 shl $10) or (SPI_IOC_MODE_CTRL shl $08) or ($00);      { $40206B00; }
  SPI_IOC_RD_MODE32 : U_Int_32        = (SPI_IOC_MODE_READ shl $18) or ($04 shl $10) or (SPI_IOC_MODE_CTRL shl $08) or ($05);       { $80046B05; }
  SPI_IOC_WR_MODE32 : U_Int_32        = (SPI_IOC_MODE_WRITE shl $18) or ($04 shl $10) or (SPI_IOC_MODE_CTRL shl $08) or ($05);      { $40046B05; }

  { ####################################################################################### }
  { ## SPI modes                                                                         ## }
  { ####################################################################################### }
  SPI_IOC_CPHA      = $01;  { Clock phase }
  SPI_IOC_CPOL      = $02;  { Clock polarity }
  SPI_IOC_CS_HIGH   = $04;  { Chip Select active high }
  SPI_IOC_LSB_FIRST = $08;  { Least significant bit first }
  SPI_IOC_3WIRE     = $10;  { SI/SO signals shared }
  SPI_IOC_LOOP      = $20;  { Loopback }
  SPI_IOC_NO_CS     = $40;  { no Chip Select }
  SPI_IOC_READY     = $80;  { Slave pulls low to pause }

  SPI_IOC_MODE_0 = (0 * SPI_IOC_CPOL) or (0 * SPI_IOC_CPHA);
  { Clock idle low, data is clocked in on rising edge, output data change on falling edge }
  SPI_IOC_MODE_1 = (0 * SPI_IOC_CPOL) or (1 * SPI_IOC_CPHA);
  { Clock idle low, data is clocked in on falling edge, output data change on rising edge }
  SPI_IOC_MODE_2 = (1 * SPI_IOC_CPOL) or (0 * SPI_IOC_CPHA);
  { Clock idle high, data is clocked in on falling edge, output data change on rising edge }
  SPI_IOC_MODE_3 = (1 * SPI_IOC_CPOL) or (1 * SPI_IOC_CPHA);
  { Clock idle high, data is clocked in on rising, edge output data change on falling edge }

  { ####################################################################################### }
  { ## SPI settings                                                                      ## }
  { ####################################################################################### }
  SPI_IOC_DEFAULT_MODE          = SPI_IOC_MODE_1;
  SPI_IOC_DEFAULT_BITS_PER_WORD = 8;
  SPI_IOC_DEFAULT_LSB_FIRST     = FALSE;
  SPI_IOC_DEFAULT_SPEED         = 8 * 1000 * 1000;

  { ####################################################################################### }
  { ## ADS1262 registers                                                                 ## }
  { ####################################################################################### }
  ADS1262_REGISTER_ID        = $00;
  ADS1262_REGISTER_POWER     = $01;
  ADS1262_REGISTER_INTERFACE = $02;
  ADS1262_REGISTER_MODE0     = $03;
  ADS1262_REGISTER_MODE1     = $04;
  ADS1262_REGISTER_MODE2     = $05;
  ADS1262_REGISTER_INPMUX    = $06;
  ADS1262_REGISTER_OFCAL0    = $07;
  ADS1262_REGISTER_OFCAL1    = $08;
  ADS1262_REGISTER_OFCAL2    = $09;
  ADS1262_REGISTER_FSCAL0    = $0A;
  ADS1262_REGISTER_FSCAL1    = $0B;
  ADS1262_REGISTER_FSCAL2    = $0C;
  ADS1262_REGISTER_IDACMUX   = $0D;
  ADS1262_REGISTER_IDACMAG   = $0E;
  ADS1262_REGISTER_REFMUX    = $0F;
  ADS1262_REGISTER_TDACP     = $10;
  ADS1262_REGISTER_TDACN     = $11;
  ADS1262_REGISTER_GPIOCON   = $12;
  ADS1262_REGISTER_GPIODIR   = $13;
  ADS1262_REGISTER_GPIODAT   = $14;

  { ####################################################################################### }
  { ## ADS1262 commands                                                                  ## }
  { ####################################################################################### }
  ADS1262_COMMAND_NOP     = $00;
  ADS1262_COMMAND_RESET   = $06;
  ADS1262_COMMAND_START1  = $08;
  ADS1262_COMMAND_STOP1   = $0A;
  ADS1262_COMMAND_RDATA1  = $12;
  ADS1262_COMMAND_SYOCAL1 = $16;
  ADS1262_COMMAND_SYGCAL1 = $17;
  ADS1262_COMMAND_SFOCAL1 = $19;
  ADS1262_COMMAND_RREG    = $20;
  ADS1262_COMMAND_WREG    = $40;

  { ####################################################################################### }
  { ## ADS1262 POWER bits                                                                ## }
  { ####################################################################################### }
  ADS1262_POWER_RESET_MASK        = $10;
  ADS1262_POWER_VBIAS_MASK        = $02;
  ADS1262_POWER_INTREF_MASK       = $01;
  ADS1262_POWER_RESET_HAS_OCCURED = $10;
  ADS1262_POWER_VBIAS_ENABLED     = $02;
  ADS1262_POWER_VBIAS_DISABLED    = $00;
  ADS1262_POWER_INTREF_ENABLED    = $01;
  ADS1262_POWER_INTREF_DISABLED   = $00;

  { ####################################################################################### }
  { ## ADS1262 MODE0 bits                                                                ## }
  { ####################################################################################### }
  ADS1262_MODE0_REFREV_MASK                                    = $80;
  ADS1262_MODE0_RUNMODE_MASK                                   = $40;
  ADS1262_MODE0_CHOP_MASK                                      = $30;
  ADS1262_MODE0_DELAY_MASK                                     = $0F;
  ADS1262_MODE0_REFREV_NORMAL_POLARITY                         = $00;
  ADS1262_MODE0_RUNMODE_CONTINUOUS_CONVERSION                  = $00;
  ADS1262_MODE0_RUNMODE_PULSE_CONVERSION                       = $40;
  ADS1262_MODE0_CHOP_INPUT_CHOP_AND_IDAC_ROTATION_DISABLED     = $00;
  ADS1262_MODE0_CHOP_INPUT_CHOP_ENABLED_IDAC_ROTATION_DISABLED = $10;
  ADS1262_MODE0_DELAY_NO_DELAY                                 = $00;

  { ####################################################################################### }
  { ## ADS1262 MODE1 bits                                                                ## }
  { ####################################################################################### }
  ADS1262_MODE1_FILTER_MASK       = $E0;
  ADS1262_MODE1_SBADC_MASK        = $10;
  ADS1262_MODE1_SBPOL_MASK        = $08;
  ADS1262_MODE1_SBMAG_MASK        = $07;
  ADS1262_MODE1_FILTER_SINC1_MODE = $00;
  ADS1262_MODE1_FILTER_SINC2_MODE = $20;
  ADS1262_MODE1_FILTER_SINC3_MODE = $40;
  ADS1262_MODE1_FILTER_SINC4_MODE = $60;
  ADS1262_MODE1_FILTER_FIR_MODE   = $80;

  { ####################################################################################### }
  { ## ADS1262 MODE2 bits                                                                ## }
  { ####################################################################################### }
  ADS1262_MODE2_BYPASS_MASK         = $80;
  ADS1262_MODE2_GAIN_MASK           = $70;
  ADS1262_MODE2_DR_MASK             = $0F;
  ADS1262_MODE2_BYPASS_PGA_BYPASSED = $80;
  ADS1262_MODE2_BYPASS_PGA_ENABLED  = $00;
  ADS1262_MODE2_GAIN_01             = $00;  { Gain  1 : ±2.500 V }
  ADS1262_MODE2_GAIN_02             = $10;  { Gain  2 : ±1.250 V }
  ADS1262_MODE2_GAIN_04             = $20;  { Gain  4 : ±0.625 V }
  ADS1262_MODE2_GAIN_08             = $30;  { Gain  8 : ±0.312 V }
  ADS1262_MODE2_GAIN_16             = $40;  { Gain 16 : ±0.156 V }
  ADS1262_MODE2_GAIN_32             = $50;  { Gain 32 : ±0.078 V }
  ADS1262_MODE2_GAIN_01_LIMIT       = 2.5;
  ADS1262_MODE2_GAIN_02_LIMIT       = ADS1262_MODE2_GAIN_01_LIMIT / 2;
  ADS1262_MODE2_GAIN_04_LIMIT       = ADS1262_MODE2_GAIN_01_LIMIT / 4;
  ADS1262_MODE2_GAIN_08_LIMIT       = ADS1262_MODE2_GAIN_01_LIMIT / 8;
  ADS1262_MODE2_GAIN_16_LIMIT       = ADS1262_MODE2_GAIN_01_LIMIT / 16;
  ADS1262_MODE2_GAIN_32_LIMIT       = ADS1262_MODE2_GAIN_01_LIMIT / 32;
  ADS1262_MODE2_DR_2_5_SPS          = $00;  { Data rate : 2.5 SPS }
  ADS1262_MODE2_DR_5_SPS            = $01;  { Data rate : 5 SPS }
  ADS1262_MODE2_DR_10_SPS           = $02;  { Data rate : 10 SPS }
  ADS1262_MODE2_DR_16_6_SPS         = $03;  { Data rate : 16.6 SPS }
  ADS1262_MODE2_DR_20_SPS           = $04;  { Data rate : 20 SPS (old default) }
  ADS1262_MODE2_DR_50_SPS           = $05;  { Data rate : 50 SPS }
  ADS1262_MODE2_DR_60_SPS           = $06;  { Data rate : 60 SPS }
  ADS1262_MODE2_DR_100_SPS          = $07;  { Data rate : 100 SPS }
  ADS1262_MODE2_DR_400_SPS          = $08;  { Data rate : 400 SPS }
  ADS1262_MODE2_DR_1200_SPS         = $09;  { Data rate : 1,200 SPS }
  ADS1262_MODE2_DR_2400_SPS         = $0A;  { Data rate : 2,400 SPS }
  ADS1262_MODE2_DR_4800_SPS         = $0B;  { Data rate : 4,800 SPS }
  ADS1262_MODE2_DR_7200_SPS         = $0C;  { Data rate : 7,200 SPS }
  ADS1262_MODE2_DR_14400_SPS        = $0D;  { Data rate : 14,400 SPS }
  ADS1262_MODE2_DR_19200_SPS        = $0E;  { Data rate : 19,200 SPS }
  ADS1262_MODE2_DR_38400_SPS        = $0F;  { Data rate : 38,400 SPS }

  { ####################################################################################### }
  { ## ADS1262 INPMUX bits                                                               ## }
  { ####################################################################################### }
  ADS1262_INPMUX_MUXP_MASK       = $F0;
  ADS1262_INPMUX_MUXN_MASK       = $0F;
  ADS1262_INPMUX_MUXP_AIN0       = $00;  { Default }
  ADS1262_INPMUX_MUXP_AIN1       = $10;
  ADS1262_INPMUX_MUXP_AIN2       = $20;
  ADS1262_INPMUX_MUXP_AIN3       = $30;
  ADS1262_INPMUX_MUXP_AIN4       = $40;
  ADS1262_INPMUX_MUXP_AIN5       = $50;
  ADS1262_INPMUX_MUXP_AIN6       = $60;
  ADS1262_INPMUX_MUXP_AIN7       = $70;
  ADS1262_INPMUX_MUXP_AIN8       = $80;
  ADS1262_INPMUX_MUXP_AIN9       = $90;
  ADS1262_INPMUX_MUXP_AINCOM     = $A0;
  ADS1262_INPMUX_MUXP_TEMP_M_POS = $B0;
  ADS1262_INPMUX_MUXP_APS_M_POS  = $C0;
  ADS1262_INPMUX_MUXP_DPS_M_POS  = $D0;
  ADS1262_INPMUX_MUXP_TDAC_POS   = $E0;
  ADS1262_INPMUX_MUXP_FLOAT      = $F0;
  ADS1262_INPMUX_MUXN_AIN0       = $00;
  ADS1262_INPMUX_MUXN_AIN1       = $01;  { Old Default }
  ADS1262_INPMUX_MUXN_AIN2       = $02;
  ADS1262_INPMUX_MUXN_AIN3       = $03;
  ADS1262_INPMUX_MUXN_AIN4       = $04;
  ADS1262_INPMUX_MUXN_AIN5       = $05;
  ADS1262_INPMUX_MUXN_AIN6       = $06;
  ADS1262_INPMUX_MUXN_AIN7       = $07;
  ADS1262_INPMUX_MUXN_AIN8       = $08;
  ADS1262_INPMUX_MUXN_AIN9       = $09;
  ADS1262_INPMUX_MUXN_AINCOM     = $0A;  { New Default }
  ADS1262_INPMUX_MUXN_TEMP_M_POS = $0B;
  ADS1262_INPMUX_MUXN_APS_M_POS  = $0C;
  ADS1262_INPMUX_MUXN_DPS_M_POS  = $0D;
  ADS1262_INPMUX_MUXN_TDAC_POS   = $0E;
  ADS1262_INPMUX_MUXN_FLOAT      = $0F;

  { ####################################################################################### }
  { ## ADS1262 initial data                                                              ## }
  { ####################################################################################### }
  ADS1262_REGISTER_OFCAL_LENGTH                = 3;
  ADS1262_WAIT_TIME_MS_COMMAND                 = 100;
  ADS1262_WAIT_TIME_MS_SELF_CALIBRATION_BEFORE = 1000;
  ADS1262_WAIT_TIME_MS_SELF_CALIBRATION_AFTER  = 100;
  ADS1262_WAIT_TIME_MS_READ_REGISTER           = 0;
  ADS1262_WAIT_TIME_MS_WRITE_REGISTER          = 10;
  ADS1262_WAIT_TIME_MS_READ_ADC1               = 0;
  ADS1262_CRC_MAGIC_BYTE                       = $9B;
  ADS1262_DATA_EMA_SPAN                        = 4000;
  ADS1262_BIT_LEVEL_EMA_SPAN                   = 100;
  ADS1262_INPMUX_MUX_P                         = ADS1262_INPMUX_MUXP_AIN0;
  ADS1262_INPMUX_MUX_N                         = ADS1262_INPMUX_MUXN_AINCOM;
  ADS1262_VBIAS_MODE                           = ADS1262_POWER_VBIAS_DISABLED;
  ADS1262_PGA_MODE                             = ADS1262_MODE2_BYPASS_PGA_ENABLED;
  ADS1262_DATA_RATE                            = ADS1262_MODE2_DR_400_SPS;
  ADS1262_FILTER_MODE                          = ADS1262_MODE1_FILTER_SINC4_MODE;
  ADS1262_CHOP_MODE                            = ADS1262_MODE0_CHOP_INPUT_CHOP_AND_IDAC_ROTATION_DISABLED;
  ADS1262_GAIN                                 = ADS1262_MODE2_GAIN_32;
  ADS1262_INITIAL_OFCAL                        = $0003E0; { 24 bit value : $000000 }
  ADS1262_INITIAL_SENSIVITY                    = 32;

Type
  { ####################################################################################### }
  { ## T_GPIO_Pin                                                                        ## }
  { ####################################################################################### }
  T_GPIO_Pin = Class(TObject)
  Protected
      M_GPIO_Pin : U_Int_32;
      Procedure GPIO_Write_File (Const F_Filename : String; Const F_Output_Buffer_A : T_Buffer_A; F_Length : U_Int_32);
      Function GPIO_Read_File (Const F_Filename : String; F_Length : U_Int_32) : String;
      Procedure Set_Export (F_Export_F : Boolean);
      Function Get_Direction () : T_GPIO_Direction;
      Procedure Set_Direction (F_GPIO_Direction : T_GPIO_Direction);
      Function Get_Value () : Boolean;
      Procedure Set_Value (F_Value : Boolean);
      Function Get_Interrupt_Mode () : T_GPIO_Interrupt_Mode;
      Procedure Set_Interrupt_Mode (F_GPIO_Interrupt_Mode : T_GPIO_Interrupt_Mode);
  Public
      Constructor Create (F_GPIO_Pin : U_Int_32);
      Destructor Destroy; Override;
      Function Wait_For_Interrupt (F_Timeout : Integer) : Boolean;
  Published
      Property Direction : T_GPIO_Direction Read Get_Direction Write Set_Direction;
      Property Value : Boolean Read Get_Value Write Set_Value;
      Property Interrupt_Mode : T_GPIO_Interrupt_Mode Read Get_Interrupt_Mode Write Set_Interrupt_Mode;
  End; { T_SPI_DEVICE }

  { ####################################################################################### }
  { ## T_SPI_Device                                                                      ## }
  { ####################################################################################### }
  T_SPI_Device = Class(TObject)
  Protected
      M_SPI_File_Handle :   Int_32;
      M_SPI_Mode :          U_Int_8;
      M_SPI_Bits_per_Word : U_Int_8;
      M_SPI_LSB_First :     Boolean;
      M_SPI_Max_Speed :     U_Int_32;
      Function Get_SPI_Mode : U_Int_8;
      Procedure Set_SPI_Mode (F_Mode : U_Int_8);
      Function Get_SPI_Bits_per_Word : U_Int_8;
      Procedure Set_SPI_Bits_per_Word (F_Bits_per_Word : U_Int_8);
      Function Get_SPI_LSB_First : Boolean;
      Procedure Set_SPI_LSB_First (F_LSB_First : Boolean);
      Function Get_SPI_Max_Speed : U_Int_32;
      Procedure Set_SPI_Max_Speed (F_Max_Speed : U_Int_32);
      Function SPI_Write_Read_Buffer (Var F_Output_Buffer_A : T_Buffer_A; Var F_Input_Buffer_A : T_Buffer_A; F_Length : U_Int_8) : Int_32;
  Public
      Constructor Create ();
      Destructor Destroy; Override;
  Published
      Property SPI_Mode : U_Int_8 Read Get_SPI_Mode Write Set_SPI_Mode;
      Property SPI_Bits_per_Word : U_Int_8 Read Get_SPI_Bits_per_Word Write Set_SPI_Bits_per_Word;
      Property SPI_LSB_First : Boolean Read Get_SPI_LSB_First Write Set_SPI_LSB_First;
      Property SPI_Max_Speed : U_Int_32 Read Get_SPI_Max_Speed Write Set_SPI_Max_Speed;
  End; { T_SPI_Device }


  { ####################################################################################### }
  { ## T_ADS1262                                                                         ## }
  { ####################################################################################### }
  T_ADS1262 = Class(T_SPI_Device)
  Protected
      M_Count :         UInt64;
      M_Bit_N :         Integer;
      M_EMA_Data :      Int_32;
      M_EMA_Bit_N :     Double;
      M_DRDY_Pin :      T_GPIO_Pin;
      M_Voltage_Limit : Double;
      M_Sensivity :     Int_32;
  Public
      Constructor Create ();
      Destructor Destroy (); Override;
      Procedure Reset_and_Init ();
      Procedure Execute_Command (F_Command : U_Int_8; F_Wait_Time_msec : Int_32 = ADS1262_WAIT_TIME_MS_COMMAND);
      Function Read_Register (F_Register_Adress : U_Int_8) : U_Int_8;
      Procedure Write_Register (F_Register_Adress : U_Int_8; F_Data : U_Int_8);
      Procedure System_Offset_Self_Calibration ();
      Procedure Set_Continuous_Conversion ();
      Procedure Set_Pulse_Conversion ();
      Procedure Start_Conversion ();
      Procedure Stop_Conversion ();
      Procedure Set_POWER (F_VBIAS_Mode : U_Int_8; F_INTREF_Mode : U_Int_8);
      Function Get_INPMUX () : U_Int_8;
      Procedure Set_INPMUX (F_MUXP : U_Int_8; F_MUXN : U_Int_8);
      Procedure Set_INPMUX_Without_Wait (F_MUXP : U_Int_8; F_MUXN : U_Int_8);
      Procedure Set_PGA (F_Bypass_Mode : U_Int_8; F_Gain_Mode : U_Int_8);
      Procedure Set_Data_Rate (F_Rate_Mode : U_Int_8);
      Procedure Set_Filter_Mode (F_Filter_Mode : U_Int_8);
      Procedure Set_Chop_Mode (F_Chop_Mode : U_Int_8);
      Procedure Set_OFCAL (F_OFCAL : Int_32);
      Function Read_ADC1_Data (Var F_Status : U_Int_8; Var F_CRC : U_Int_8) : Int_32;
      Function Read_ADC1_Data : Int_32;
      Procedure Read_All_Registers (Var F_Register_Block : T_Register_Block);
      Function Calculate_Voltage (F_Data : Int_32) : Double;
      Function Get_Voltage () : Double;
      Function Calculate_Sensivity_Value (F_Value : Int_32; F_Sensivity : Int_32) : Int_32;
  Published
      Property DRDY_Pin : T_GPIO_Pin Read M_DRDY_Pin;
      Property ADC1_Data : Int_32 Read Read_ADC1_Data;
      Property Voltage : Double Read Get_Voltage;
      Property EMA_Data : Int_32 Read M_EMA_Data;
      Property EMA_Bit_N : Double Read M_EMA_Bit_N;
      Property Sensivity : Int_32 Read M_Sensivity Write M_Sensivity;
  End; { T_ADS1262 }


Implementation

Uses
  Math,
  SysUtils,
  Dialogs,
  DateUtils;


{ ####################################################################################### }
{ ## T_GPIO_Pin                                                                        ## }
{ ####################################################################################### }

{ --------------------------------------------------------------------------------------- }
Constructor T_GPIO_Pin.Create (F_GPIO_Pin : U_Int_32);
  { Initialization of the object                                                            }
  { --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.Create }
  Inherited Create ();
End; { T_GPIO_Pin.Create }


{ --------------------------------------------------------------------------------------- }
Destructor T_GPIO_Pin.Destroy ();
  { Free data                                                                               }
  { --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.Destroy }
  Inherited Destroy;
End; { T_GPIO_Pin.Destroy }


{ --------------------------------------------------------------------------------------- }
Procedure T_GPIO_Pin.GPIO_Write_File (Const F_Filename : String; Const F_Output_Buffer_A : T_Buffer_A; F_Length : U_Int_32);
{ Write data to file                                                                      }
{ --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.GPIO_Write_File }
End; { T_GPIO_Pin.GPIO_Write_File }


{ --------------------------------------------------------------------------------------- }
Function T_GPIO_Pin.GPIO_Read_File (Const F_Filename : String; F_Length : U_Int_32) : String;
  { Read data from file                                                                     }
  { --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.GPIO_Read_File }
  Result := '';
End; { T_GPIO_Pin.GPIO_Read_File }


{ --------------------------------------------------------------------------------------- }
Procedure T_GPIO_Pin.Set_Export (F_Export_F : Boolean);
{ Set export                                                                              }
{ --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.Set_Export }
End; { T_GPIO_Pin.Set_Export }


{ --------------------------------------------------------------------------------------- }
Function T_GPIO_Pin.Get_Direction () : T_GPIO_Direction;
  { Get direction of pin                                                                    }
  { --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.Get_Direction }
  Result := GPIO_DIRECTION_IN;
End; { T_GPIO_Pin.Get_Direction }


{ --------------------------------------------------------------------------------------- }
Procedure T_GPIO_Pin.Set_Direction (F_GPIO_Direction : T_GPIO_Direction);
{ Get direction of pin                                                                    }
{ --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.Set_Direction }
End; { T_GPIO_Pin.Set_Direction }


{ --------------------------------------------------------------------------------------- }
Function T_GPIO_Pin.Get_Value () : Boolean;
  { Get value of pin                                                                        }
  { --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.Get_Value }
  Result := FALSE;
End; { T_GPIO_Pin.Get_Value }


{ --------------------------------------------------------------------------------------- }
Procedure T_GPIO_Pin.Set_Value (F_Value : Boolean);
{ Get value of pin                                                                        }
{ --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.Set_Value }
End; { T_GPIO_Pin.Set_Value }


{ --------------------------------------------------------------------------------------- }
Function T_GPIO_Pin.Get_Interrupt_Mode () : T_GPIO_Interrupt_Mode;
  { Get interrupt mode                                                                      }
  { --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.Get_Interrupt_Mode }
  Result := [];
End; { T_GPIO_Pin.Get_Interrupt_Mode }


{ --------------------------------------------------------------------------------------- }
Procedure T_GPIO_Pin.Set_Interrupt_Mode (F_GPIO_Interrupt_Mode : T_GPIO_Interrupt_Mode);
{ Set interrupt mode                                                                      }
{ --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.Set_Interrupt_Mode }
End; { T_GPIO_Pin.Set_Interrupt_Mode }


{ --------------------------------------------------------------------------------------- }
Function T_GPIO_Pin.Wait_For_Interrupt (F_Timeout : Integer) : Boolean;
  { Wait for next interrupt or timeout                                                      }
  { --------------------------------------------------------------------------------------- }
Begin { T_GPIO_Pin.Wait_For_Interrupt }
  Result := FALSE;
End; { T_GPIO_Pin.Wait_For_Interrupt }


{ ####################################################################################### }
{ ## T_SPI_Device                                                                      ## }
{ ####################################################################################### }

{ --------------------------------------------------------------------------------------- }
Constructor T_SPI_Device.Create ();
  { Initialization of the object                                                            }
  { --------------------------------------------------------------------------------------- }
Begin { T_SPI_Device.Create }
  Inherited Create ();
End; { T_SPI_Device.Create }


{ --------------------------------------------------------------------------------------- }
Destructor T_SPI_Device.Destroy ();
  { Free data                                                                               }
  { --------------------------------------------------------------------------------------- }
Begin { T_SPI_Device.Destroy }
  Inherited Destroy;
End; { T_SPI_Device.Destroy }


{ --------------------------------------------------------------------------------------- }
Function T_SPI_Device.Get_SPI_Mode () : U_Int_8;
  { Get SPI mode                                                                            }
  { --------------------------------------------------------------------------------------- }
Begin { T_SPI_Device.Get_SPI_Mode }
  Result := 0;
End; { T_SPI_Device.Get_SPI_Mode }


{ --------------------------------------------------------------------------------------- }
Procedure T_SPI_Device.Set_SPI_Mode (F_Mode : U_Int_8);
{ Set SPI mode                                                                            }
{ --------------------------------------------------------------------------------------- }
Begin { T_SPI_Device.Set_SPI_Mode }
End; { T_SPI_Device.Set_SPI_Mode }


{ --------------------------------------------------------------------------------------- }
Function T_SPI_Device.Get_SPI_Bits_per_Word : U_Int_8;
  { Get bits per word                                                                       }
  { --------------------------------------------------------------------------------------- }
Begin { T_SPI_Device.Get_SPI_Bits_per_Word }
  Result := 0;
End; { T_SPI_Device.Get_SPI_Bits_per_Word }


{ --------------------------------------------------------------------------------------- }
Procedure T_SPI_Device.Set_SPI_Bits_per_Word (F_Bits_per_Word : U_Int_8);
{ Set bits per word                                                                       }
{ --------------------------------------------------------------------------------------- }
Begin { T_SPI_Device.Set_SPI_Bits_per_Word }
End; { T_SPI_Device.Set_SPI_Bits_per_Word }


{ --------------------------------------------------------------------------------------- }
Function T_SPI_Device.Get_SPI_LSB_First : Boolean;
  { Get bits per word                                                                       }
  { --------------------------------------------------------------------------------------- }
Begin { T_SPI_Device.Get_SPI_LSB_First }
  Result := FALSE;
End; { T_SPI_Device.Get_SPI_LSB_First }


{ --------------------------------------------------------------------------------------- }
Procedure T_SPI_Device.Set_SPI_LSB_First (F_LSB_First : Boolean);
{ Set LSB_First; F_LSB_First=$00 or F_LSB_First=$FF                                       }
{ --------------------------------------------------------------------------------------- }
Begin { T_SPI_Device.Set_SPI_LSB_First }
End; { T_SPI_Device.Set_SPI_LSB_First }


{ --------------------------------------------------------------------------------------- }
Function T_SPI_Device.Get_SPI_Max_Speed : U_Int_32;
  { Get bits per word                                                                       }
  { --------------------------------------------------------------------------------------- }
Begin { T_SPI_Device.Get_SPI_Max_Speed }
  Result := 0;
End; { T_SPI_Device.Get_SPI_Max_Speed }


{ --------------------------------------------------------------------------------------- }
Procedure T_SPI_Device.Set_SPI_Max_Speed (F_Max_Speed : U_Int_32);
{ Set bits per word                                                                       }
{ --------------------------------------------------------------------------------------- }
Begin { T_SPI_Device.Set_SPI_Max_Speed }
End; { T_SPI_Device.Set_SPI_Max_Speed }


{ --------------------------------------------------------------------------------------- }
Function T_SPI_Device.SPI_Write_Read_Buffer (Var F_Output_Buffer_A : T_Buffer_A; Var F_Input_Buffer_A : T_Buffer_A; F_Length : U_Int_8) : Int_32;
  { Write and read buffer                                                                   }
  { --------------------------------------------------------------------------------------- }
Begin { T_SPI_Device.SPI_Write_Read_Buffer }
  Result := 0;
End;       { T_SPI_Device.SPI_Write_Read_Buffer }


{ ####################################################################################### }
{ ## T_ADS1262                                                                         ## }
{ ####################################################################################### }

{ --------------------------------------------------------------------------------------- }
Constructor T_ADS1262.Create ();
  { Initialization of the object                                                            }
  { --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Create }
  Inherited Create ();

  M_Sensivity := ADS1262_INITIAL_SENSIVITY;
End; { T_ADS1262.Create }


{ --------------------------------------------------------------------------------------- }
Destructor T_ADS1262.Destroy ();
  { Free data                                                                               }
  { --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Destroy }
  Inherited Destroy ();
End; { T_ADS1262.Destroy }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Reset_and_Init ();
{ Init ADC                                                                                }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Reset_and_Init }
End; { T_ADS1262.Reset_and_Init }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Execute_Command (F_Command : U_Int_8; F_Wait_Time_msec : Int_32 = ADS1262_WAIT_TIME_MS_COMMAND);
{ Execute a one byte command                                                              }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Execute_Command }
End;     { T_ADS1262.Execute_Command }


{ --------------------------------------------------------------------------------------- }
Function T_ADS1262.Read_Register (F_Register_Adress : U_Int_8) : U_Int_8;
  { Read data from register                                                                 }
  { --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Read_Register }
  Result := 0;
End; { T_ADS1262.Read_Register }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Write_Register (F_Register_Adress : U_Int_8; F_Data : U_Int_8);
{ Write data into register                                                                }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Write_Register }
End; { T_ADS1262.Write_Register }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.System_Offset_Self_Calibration ();
{ ADC1 self offset calibration SFOCAL1                                                    }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.System_Offset_Self_Calibration }
End; { T_ADS1262.System_Offset_Self_Calibration }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Set_Continuous_Conversion ();
{ Start continuous conversion                                                             }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Set_Continuous_Conversion }
End; { T_ADS1262.Set_Continuous_Conversion }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Set_Pulse_Conversion ();
{ Start pulse conversion                                                                  }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Set_Pulse_Conversion }
End; { T_ADS1262.Set_Pulse_Conversion }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Start_Conversion ();
{ Start conversion                                                                        }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Start_Conversion }
End; { T_ADS1262.Start_Conversion }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Stop_Conversion ();
{ Stop conversion                                                                         }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Stop_Conversion }
End; { T_ADS1262.Stop_Conversion }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Set_POWER (F_VBIAS_Mode : U_Int_8; F_INTREF_Mode : U_Int_8);
{ Set POWER mode                                                                          }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Set_POWER }
End; { T_ADS1262.Set_POWER }


{ --------------------------------------------------------------------------------------- }
Function T_ADS1262.Get_INPMUX () : U_Int_8;
  { Get INPMUX                                                                              }
  { --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Get_INPMUX }
  Result := 0;
End; { T_ADS1262.Get_INPMUX }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Set_INPMUX (F_MUXP : U_Int_8; F_MUXN : U_Int_8);
{ Set INPMUX                                                                              }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Set_INPMUX }
End; { T_ADS1262.Set_INPMUX }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Set_INPMUX_Without_Wait (F_MUXP : U_Int_8; F_MUXN : U_Int_8);
{ Set INPMUX                                                                              }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Set_INPMUX_Without_Wait }
End; { T_ADS1262.Set_INPMUX_Without_Wait }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Set_PGA (F_Bypass_Mode : U_Int_8; F_Gain_Mode : U_Int_8);
{ Set PGA mode                                                                            }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Set_PGA }
End; { T_ADS1262.Set_PGA }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Set_Data_Rate (F_Rate_Mode : U_Int_8);
{ Set data rate                                                                           }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Set_Data_Rate }
End; { T_ADS1262.Set_Data_Rate }

{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Set_Filter_Mode (F_Filter_Mode : U_Int_8);
{ Set filter mode                                                                         }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Set_Filter_Mode }
End; { T_ADS1262.Set_Filter_Mode }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Set_Chop_Mode (F_Chop_Mode : U_Int_8);
{ Set chop mode                                                                           }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Set_Chop_Mode }
End; { T_ADS1262.Set_Chop_Mode }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Set_OFCAL (F_OFCAL : Int_32);
{ Set offset calibration registers                                                        }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Set_OFCAL }
End; { T_ADS1262.Set_OFCAL }


{ --------------------------------------------------------------------------------------- }
Function T_ADS1262.Read_ADC1_Data (Var F_Status : U_Int_8; Var F_CRC : U_Int_8) : Int_32;
  { Read ADC1 data                                                                          }
  { --------------------------------------------------------------------------------------- }
Var
  EMA_Factor :      Double;

Begin { T_ADS1262.Read_ADC1_Data }
  Result := MilliSecondOf (Now) + (SecondOf (Now) * 1000) + (MinuteOf (Now) * 1000 * 60);
  Result := Round (Sin (Result / 100) * 100000.1);

  EMA_Factor := 2 / (ADS1262_DATA_EMA_SPAN + 1);
  M_EMA_Data := Round ((EMA_Factor * Result) + ((1 - EMA_Factor) * M_EMA_Data));

  EMA_Factor  := 2 / (ADS1262_BIT_LEVEL_EMA_SPAN + 1);
  M_Bit_N     := Round (Int (Log2 (Max (Abs (Result), 1))));
  M_EMA_Bit_N := (EMA_Factor * M_Bit_N) + ((1 - EMA_Factor) * M_EMA_Bit_N);
End; { T_ADS1262.Read_ADC1_Data }


{ --------------------------------------------------------------------------------------- }
Function T_ADS1262.Read_ADC1_Data : Int_32;
  { Read ADC1 data                                                                          }
  { --------------------------------------------------------------------------------------- }
Var
  Status : U_Int_8;
  CRC :    U_Int_8;

Begin { T_ADS1262.Read_ADC1_Data }
  { Check for recalibration }
  Result := Read_ADC1_Data (Status, CRC);
End; { T_ADS1262.Read_ADC1_Data }


{ --------------------------------------------------------------------------------------- }
Procedure T_ADS1262.Read_All_Registers (Var F_Register_Block : T_Register_Block);
{ Read all registers into register array                                                  }
{ --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Read_All_Registers }
End; { T_ADS1262.Read_All_Registers }


{ --------------------------------------------------------------------------------------- }
Function T_ADS1262.Calculate_Voltage (F_Data : Int_32) : Double;
  { Calculate voltage                                                                       }
  { --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Calculate_Voltage }
  { Calculate voltage }

  M_Voltage_Limit := 1;

  Result := Calculate_Sensivity_Value (F_Data, M_Sensivity) * M_Voltage_Limit / $7FFFFFFF;
End; { T_ADS1262.Calculate_Voltage }


{ --------------------------------------------------------------------------------------- }
Function T_ADS1262.Get_Voltage () : Double;
  { Read voltage from ADC                                                                   }
  { --------------------------------------------------------------------------------------- }
Begin { T_ADS1262.Get_Voltage }
  { Calculate voltage }
  Result := Calculate_Voltage (Read_ADC1_Data);
End; { T_ADS1262.Get_Voltage }


{ --------------------------------------------------------------------------------------- }
Function T_ADS1262.Calculate_Sensivity_Value (F_Value : Int_32; F_Sensivity : Int_32) : Int_32;
  { --------------------------------------------------------------------------------------- }
Var
  Mask : Int_32;

Begin { T_ADS1262.Calculate_Sensivity_Value }
  If F_Value >= 0 Then
    Begin { then }
      If F_Sensivity < 32 Then
        Begin { then }
          Mask   := not (($01 shl (32 - F_Sensivity)) - 1);
          Result := F_Value and Mask;
        End { then }
      Else
        Begin { else  }
          Result := F_Value;
        End; { else  }
    End { then }
  Else
    Begin { else  }
      If F_Sensivity < 32 Then
        Begin { then }
          Mask   := not (($01 shl (32 - F_Sensivity)) - 1);
          Result := -(-F_Value and Mask);
        End { then }
      Else
        Begin { else  }
          Result := F_Value;
        End; { else  }
    End; { else  }
End; { T_ADS1262.Calculate_Sensivity_Value }


End.
