{ ####################################################################################### }
{ ##                                                                                   ## }
{ ## Main_Unit                                                                         ## }
{ ##                                                                                   ## }
{ ## Main form for db_receiver                                                         ## }
{ ##                                                                                   ## }
{ ## Copyright (C) 2018-2019  : Dr. Jürgen Abel                                        ## }
{ ## Email                    : juergen@juergen-abel.info                              ## }
{ ## Internet                 : www.seismometer.info                                   ## }
{ ##                                                                                   ## }
{ ## This program is free software: you can redistribute it and/or modify              ## }
{ ## it under the terms of the GNU Lesser General Public License as published by       ## }
{ ## the Free Software Foundation, either version 3 of the License, or                 ## }
{ ## (at your option) any later version with the following modification:               ## }
{ ##                                                                                   ## }
{ ## As a special exception, the copyright holders of this library give you            ## }
{ ## permission to link this library with independent modules to produce an            ## }
{ ## executable, regardless of the license terms of these independent modules, and     ## }
{ ## to copy and distribute the resulting executable under terms of your choice,       ## }
{ ## provided that you also meet, for each linked independent module, the terms        ## }
{ ## and conditions of the license of that module. An independent module is a          ## }
{ ## module which is not derived from or based on this library. If you modify          ## }
{ ## this library, you may extend this exception to your version of the library,       ## }
{ ## but you are not obligated to do so. If you do not wish to do so, delete this      ## }
{ ## exception statement from your version.                                            ## }
{ ##                                                                                   ## }
{ ## This program is distributed in the hope that it will be useful,                   ## }
{ ## but WITHOUT ANY WARRANTY; without even the implied warranty of                    ## }
{ ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                     ## }
{ ## GNU General Public License for more details.                                      ## }
{ ##                                                                                   ## }
{ ## You should have received a copy of the GNU Lesser General Public License          ## }
{ ## COPYING.LGPL.txt along with this program.                                         ## }
{ ## If not, see <https://www.gnu.org/licenses/>.                                      ## }
{ ##                                                                                   ## }
{ ####################################################################################### }


Unit Main_Unit;

{$mode objfpc}{$H+}

{$INCLUDE project_defines.inc}

Interface

Uses
  CThreads,
  Classes,
  SysUtils,
  Forms,
  Controls,
  Graphics,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  TAGraph,
  TASeries,
  ADS1262_Unit,
  TACustomSeries,
  TAIntervalSources,
  TATools,
  Types,
  TACustomSource,
  TAFuncSeries,
  TASources,
  TAChartExtentLink,
  SpinEx,
  PQConnection,
  SQLDB;

Const
  ADS1262_MODE2_GAIN_01_LIMIT       = 2.5;
  ADS1262_MODE2_GAIN_02_LIMIT       = ADS1262_MODE2_GAIN_01_LIMIT / 2;
  ADS1262_MODE2_GAIN_04_LIMIT       = ADS1262_MODE2_GAIN_01_LIMIT / 4;
  ADS1262_MODE2_GAIN_08_LIMIT       = ADS1262_MODE2_GAIN_01_LIMIT / 8;
  ADS1262_MODE2_GAIN_16_LIMIT       = ADS1262_MODE2_GAIN_01_LIMIT / 16;
  ADS1262_MODE2_GAIN_32_LIMIT       = ADS1262_MODE2_GAIN_01_LIMIT / 32;
  {$IFDEF USE_INTERRUPT}
  {$ELSE}
  DATA_INTERVAL_MS                  = 2;
  {$ENDIF}
  LP_SLOW_SPAN                      = 4000;
  LP_SLOW_SETTLE_FACTOR             = 4;
  LP_INTEGRAL_SPAN                  = 1200;
  LP_INTEGRAL_CORRECTION_DIVIDER    = 40000;
  DATA_ARRAY_SIZE                   = 1024 * 1024;
  DATA_CHART_EXTEND_Y               = 0.02;
  DATA_CHART_EXTEND_Y_MAX           = 2.5;
  DATA_CHART_LABEL_SIZE             = 120;
  DATA_CHART_UNIT_FORMAT            = '%0:.9g mm/s';
  TIME_UNIT_FORMAT                  = '%0:.9g mm/s';
  VALUES_PER_HOUR                   = 60 * 60;
  RMS_COLOR                         = $2222AA;
  RMS_BAR_WIDTH_PERCENT             = 100;
  DATA_COLOR                        = $FF4444;
  DATA_PIN_WIDTH                    = 3;
  POSITION_COLOR                    = $6666EE;
  POSITION_WIDTH                    = 11;
  IMAGE_FILE_NAME                   = 'images/daily_image.jpg';
  HOUR_CHART_N                      = 24;
  HOUR_CHART_FONT_SIZE              = 6;
  STATION_NAME                      = 'NORF_00';
  CURRENT_TIME_FORMAT_00_S          = 'Live Data  -  Station Name: ' + STATION_NAME + '  -  Date: ';
  CURRENT_TIME_FORMAT_01_S          = '  -  Time: ';
  SQR_2                             = sqrt (2);
  BIT_N_RANGE                       = 100;
  VOLTAGE_LIMIT                     = ADS1262_MODE2_GAIN_32_LIMIT;
  SM24_VELOCITY_FACTOR              = 20.9;
  {$IFDEF USE_DB_LOCALHOST}
  SQL_HOSTNAME                      = '127.0.0.1';
  {$ELSE}
  SQL_HOSTNAME                      = '192.168.0.99';
  {$ENDIF}
  SQL_DATABASENAME                  = 'seismometer_db';
  SQL_USERNAME                      = 'user';
  SQL_PASSWORD                      = '123456';
  SQL_POSTGRES_USERNAME             = 'postgres';
  SQL_POSTGRES_PASSWORD             = '123456';
  SQL_RESTART_SEQUENCE_TXT          = 'ALTER SEQUENCE adc_values_id_seq RESTART WITH 0;';
  SQL_END_TRANSATION_TXT            = 'END TRANSACTION;';
  SQL_BEGIN_TRANSATION_TXT          = 'BEGIN TRANSACTION;';
  SQL_VACUUM_FULL_TXT               = 'VACUUM FULL;';
  SQL_VACUUM_ANALYZE_TXT            = 'VACUUM ANALYZE;';
  SQL_ADC_VALUES_RECORD_N           = 400 * 60 * 60 * 24 * 31;
  SQL_ADC_VALUES_RECORD_AT_ONCE_N   = 1000;
  SQL_ADC_VALUES_TABLE_NAME_TXT     = 'adc_values';
  SQL_ADC_VALUES_ID_TXT             = 'id';
  SQL_ADC_VALUES_TIME_STAMP_TXT     = 'time_stamp';
  SQL_ADC_VALUES_ADC_VALUE_TXT      = 'adc_value';
  SQL_ADC_VALUES_TRUNCATE_TXT       = 'TRUNCATE ' + SQL_ADC_VALUES_TABLE_NAME_TXT + ';';
  SQL_ADC_VALUES_INSERT_DUMMIES_TXT = 'INSERT INTO ' + SQL_ADC_VALUES_TABLE_NAME_TXT + ' (' + SQL_ADC_VALUES_TIME_STAMP_TXT + ', ' + SQL_ADC_VALUES_ADC_VALUE_TXT + ') VALUES (CURRENT_TIMESTAMP , 0);';
  SQL_ADC_VALUES_UPDATE_TXT         = 'UPDATE ' + SQL_ADC_VALUES_TABLE_NAME_TXT + ' SET ' + SQL_ADC_VALUES_TIME_STAMP_TXT + ' = :' + SQL_ADC_VALUES_TIME_STAMP_TXT + ', ' + SQL_ADC_VALUES_ADC_VALUE_TXT + ' = :' + SQL_ADC_VALUES_ADC_VALUE_TXT + ' WHERE ' + SQL_ADC_VALUES_ID_TXT + ' = :' + SQL_ADC_VALUES_ID_TXT + ';';
  SQL_ID_VALUES_TABLE_NAME_TXT      = 'id_values';
  SQL_ID_VALUES_ID_TXT              = 'id';
  SQL_ID_VALUES_ID_VALUE_TXT        = 0;
  SQL_ID_VALUES_ADC_VALUES_ID_TXT   = 'adc_values_id';
  SQL_ID_VALUES_UPDATE_TXT          = 'UPDATE ' + SQL_ID_VALUES_TABLE_NAME_TXT + ' SET ' + SQL_ID_VALUES_ADC_VALUES_ID_TXT + ' = :' + SQL_ID_VALUES_ADC_VALUES_ID_TXT + ' WHERE ' + SQL_ADC_VALUES_ID_TXT + ' = :' + SQL_ADC_VALUES_ID_TXT + ';';
  SQL_READ_ID_TXT                   = 'SELECT ' + SQL_ID_VALUES_ADC_VALUES_ID_TXT + ' from ' + SQL_ID_VALUES_TABLE_NAME_TXT + ' LIMIT 1;';
  SQL_READ_DATA_TXT                 = 'SELECT id, time_stamp, adc_value FROM adc_values WHERE (id >= %d) AND (id < %d) ORDER BY id ASC;';

Type
  T_Data_X_A = Array [0 .. DATA_ARRAY_SIZE - 1] Of TDateTime;
  T_Data_Y_A = Array [0 .. DATA_ARRAY_SIZE - 1] Of Int_32;

  { ####################################################################################### }
  { ## TMain_F                                                                           ## }
  { ####################################################################################### }

  { TMain_F }

  TMain_F = Class(TForm)
      ADC_Values_Q :                TSQLQuery;
      ID_Values_Q :                 TSQLQuery;
      Data_LS :                     TLineSeries;
      Data_Chart_C :                TChart;
      Chart_TS :                    TChartToolset;
      Chart_TS_ZoomDragTool :       TZoomDragTool;
      Chart_TS_DataPointHintTool :  TDataPointHintTool;
      Chart_TS_PanDragTool :        TPanDragTool;
      Chart_TS_ZoomMouseWheelTool : TZoomMouseWheelTool;
      Close_B :                     TButton;
      Data_CL :                     TConstantLine;
      Label_LP :                    TLabel;
      Main_PC :                     TPageControl;
      Postgres_C :                  TPQConnection;
      Postgres_T :                  TSQLTransaction;
      Main_P :                      TPanel;
      Output_1_P :                  TPanel;
      Output_2_P :                  TPanel;
      Output_Bits_E :               TEdit;
      Output_Bits_L :               TLabel;
      Output_Counter_E :            TEdit;
      Output_Counter_L :            TLabel;
      Output_Freq_E :               TEdit;
      Output_Freq_L :               TLabel;
      Output_NPS_E :                TEdit;
      Output_NPS_L :                TLabel;
      Reset_B :                     TButton;
      Start_Stop_B :                TButton;
      Data_Chart_TS :               TTabSheet;
      Status_L :                    TLabel;
      Status_P :                    TPanel;
      Time_CS :                     TDateTimeIntervalChartSource;
      Chart_T :                     TTimer;
      Procedure Chart_TS_DataPointHintToolHint (ATool : TDataPointHintTool; Const APoint : TPoint; Var AHint : String);
      Procedure Chart_TTimer (Sender : TObject);
      Procedure Close_BClick (Sender : TObject);
      Procedure FormCreate (Sender : TObject);
      Procedure FormDestroy (Sender : TObject);
      Procedure FormShow (Sender : TObject);
      Procedure Reset_BClick (Sender : TObject);
      Procedure Start_Stop_BClick (Sender : TObject);
  Protected
      M_Data_X_A :         T_Data_X_A;
      M_Data_Y_A :         T_Data_Y_A;
      M_DB_id :            Int_64;
      M_DB_Start_id :      Int_64;
      M_DB_End_id :        Int_64;
      M_Data_Start_Index : Integer;
      M_Data_End_Index :   Integer;
      M_Data_A_Size :      Integer;
      M_Start_Time :       TDateTime;
      Procedure Show_Status (F_Status : String);
      Function Validate_Array_Index (F_Index : Integer; F_Array_Size : Integer) : Integer;
      Function Calculate_Velocity (F_Data : Int_32) : Double;
      Procedure Read_Data;
      Procedure Output_Current_Data (F_Measurement_N : Integer; F_Zero_Crossings_N : Integer; F_End_Time : TDateTime);
  Public
  Published
  End; { TMain_F }

Var
  Main_F : TMain_F;

Implementation

{$R *.frm}

Uses
  DB,
  Math,
  DateUtils,
  TAChartUtils;

{ ####################################################################################### }
{ ## TMain_F                                                                           ## }
{ ####################################################################################### }

{#####################################################################################}
{ Build form                                                                          }
{#####################################################################################}

{ --------------------------------------------------------------------------------------- }
Procedure TMain_F.FormCreate (Sender : TObject);
{ Create main from                                                                        }
{ --------------------------------------------------------------------------------------- }
Begin { TMain_F.FormCreate }
  FillByte (M_Data_X_A [0], DATA_ARRAY_SIZE * SizeOf (TDateTime), $00);
  FillByte (M_Data_Y_A [0], DATA_ARRAY_SIZE * SizeOf (Int_32), $00);

  { Initialize current chart }
  Data_LS.Clear;

  Data_Chart_C.Extent.UseXMin          := TRUE;
  Data_Chart_C.Extent.UseXMax          := TRUE;
  Data_Chart_C.Extent.UseYMin          := TRUE;
  Data_Chart_C.Extent.UseYMax          := TRUE;
  Data_Chart_C.ExtentSizeLimit.YMax    := DATA_CHART_EXTEND_Y_MAX * 2;
  Data_Chart_C.ExtentSizeLimit.YMin    := -DATA_CHART_EXTEND_Y_MAX * 2;
  Data_Chart_C.ExtentSizeLimit.UseYMax := TRUE;
  Data_Chart_C.ExtentSizeLimit.UseYMin := TRUE;

  Data_Chart_C.AxisList.Axes[0].LabelSize    := DATA_CHART_LABEL_SIZE;
  Data_Chart_C.AxisList.Axes[0].Marks.Format := DATA_CHART_UNIT_FORMAT;

  M_DB_Start_id      := 0;
  M_DB_End_id        := 0;
  M_Data_Start_Index := 0;
  M_Data_End_Index   := 0;

  Reset_BClick (Sender);
End; { TMain_F.FormCreate }


{ --------------------------------------------------------------------------------------- }
Procedure TMain_F.FormDestroy (Sender : TObject);
{ Free data                                                                               }
{ --------------------------------------------------------------------------------------- }
Begin { TMain_F.FormDestroy }
  Chart_T.Enabled := FALSE;

  Postgres_C.Close;
End; { TMain_F.FormDestroy }


{ --------------------------------------------------------------------------------------- }
Procedure TMain_F.FormShow (Sender : TObject);
{ Show main form                                                                          }
{ --------------------------------------------------------------------------------------- }
Begin { TMain_F.FormShow }
  ADC_Values_Q.Active  := FALSE;
  ID_Values_Q.Active   := FALSE;
  Postgres_T.Active    := FALSE;
  Postgres_C.Connected := FALSE;

  Postgres_C.Transaction  := Postgres_T;
  Postgres_C.HostName     := SQL_HOSTNAME;
  Postgres_C.DatabaseName := SQL_DATABASENAME;
  Postgres_C.UserName     := SQL_USERNAME;
  Postgres_C.Password     := SQL_PASSWORD;
  Postgres_C.Connected    := TRUE;

  ADC_Values_Q.Close;
  ADC_Values_Q.SQL.Clear;

  ID_Values_Q.Close;
  ID_Values_Q.SQL.Clear;
  ID_Values_Q.SQL.Add (SQL_READ_ID_TXT);

  { Read last id from db }
  ID_Values_Q.Open;
  M_DB_Start_id := ID_Values_Q.Fields [0].AsLargeInt;
  ID_Values_Q.Close;

  { Start timer }
  Chart_T.Enabled := TRUE;

  Application.ProcessMessages;
End; { TMain_F.FormShow }


{#####################################################################################}
{ Buttons and edit components                                                         }
{#####################################################################################}

{ --------------------------------------------------------------------------------------- }
Procedure TMain_F.Show_Status (F_Status : String);
{ Show status and update form                                                             }
{ --------------------------------------------------------------------------------------- }
Begin { TMain_F.Show_Status }
  Status_L.Caption := F_Status;

  Update;
  Application.ProcessMessages;
End; { TMain_F.Show_Status }


{ --------------------------------------------------------------------------------------- }
Procedure TMain_F.Output_Current_Data (F_Measurement_N : Integer; F_Zero_Crossings_N : Integer; F_End_Time : TDateTime);
{ --------------------------------------------------------------------------------------- }
Var
  I :         Integer;
  Bit_N_Max : Double;

Begin { TMain_F.Output_Current_Data }
  Bit_N_Max := 0;
  For I := 0 To BIT_N_RANGE - 1 Do
    Begin { For }
      Bit_N_Max := Max (Bit_N_Max, Log2 (Max (Abs (M_Data_Y_A [Validate_Array_Index (M_Data_A_Size - 1 - I, DATA_ARRAY_SIZE)]), 1)));
    End; { For }

  Output_NPS_E.Text     := FormatFloat ('#,##0.0', Double (F_Measurement_N) / (Double (MilliSecondSpan (M_Start_Time, F_End_Time)) / 1000));
  Output_Counter_E.Text := FormatFloat ('#,##0', M_DB_id);
  Output_Bits_E.Text    := FormatFloat ('#,##0.0', Bit_N_Max);
  Output_Freq_E.Text    := FormatFloat ('#,##0.0', F_Zero_Crossings_N / 2 / Double (MilliSecondSpan (M_Start_Time, F_End_Time)) * 1000);
  M_Start_Time          := F_End_Time;
End; { TMain_F.Output_Current_Data }


{ --------------------------------------------------------------------------------------- }
Procedure TMain_F.Close_BClick (Sender : TObject);
{ Close button pressed                                                                    }
{ --------------------------------------------------------------------------------------- }
Begin { TMain_F.Close_BClick }
  Close;
End; { TMain_F.Close_BClick }


{ --------------------------------------------------------------------------------------- }
Procedure TMain_F.Start_Stop_BClick (Sender : TObject);
{ Start/Stop button pressed                                                               }
{ --------------------------------------------------------------------------------------- }
Begin { TMain_F.Start_Stop_BClick }
  Chart_T.Enabled := not Chart_T.Enabled;
End; { TMain_F.Start_Stop_BClick }


{ --------------------------------------------------------------------------------------- }
Procedure TMain_F.Reset_BClick (Sender : TObject);
{ --------------------------------------------------------------------------------------- }
Begin { TMain_F.Reset_BClick }
  Data_Chart_C.Extent.XMin := Now - (60 / 24 / 60 / 60);
  Data_Chart_C.Extent.XMax := Now;
  Data_Chart_C.Extent.YMax := DATA_CHART_EXTEND_Y;
  Data_Chart_C.Extent.YMin := -DATA_CHART_EXTEND_Y;
End; { TMain_F.Reset_BClick }


{#####################################################################################}
{ Utilities                                                                           }
{#####################################################################################}

{ --------------------------------------------------------------------------------------- }
Function TMain_F.Validate_Array_Index (F_Index : Integer; F_Array_Size : Integer) : Integer;
  { Validate index into array                                                               }
  { --------------------------------------------------------------------------------------- }
Begin { TMain_F.Validate_Array_Index }
  If F_Index >= 0 Then
    Begin { then }
      Result := F_Index mod F_Array_Size;
    End { then }
  Else
    Begin { else  }
      Result := ((F_Index mod F_Array_Size) + F_Array_Size) mod F_Array_Size;
    End; { else  }
End; { TMain_F.Validate_Array_Index }


{ --------------------------------------------------------------------------------------- }
Function TMain_F.Calculate_Velocity (F_Data : Int_32) : Double;
  { Calculate voltage from raw data                                                         }
  { --------------------------------------------------------------------------------------- }
Begin { TMain_F.Calculate_Velocity }
  Result := F_Data * VOLTAGE_LIMIT / $7FFFFFFF / SM24_VELOCITY_FACTOR * 1000;
End; { TMain_F.Calculate_Velocity }


{#####################################################################################}
{ DB data                                                                             }
{#####################################################################################}

{ --------------------------------------------------------------------------------------- }
Procedure TMain_F.Read_Data;
{ Read current data from db                                                               }
{ --------------------------------------------------------------------------------------- }
Begin { TMain_F.Read_Data }
  { Read last id from db }
  ID_Values_Q.Open;
  M_DB_End_id := ID_Values_Q.Fields [0].AsLargeInt;
  ID_Values_Q.Close;

  { Read data }
  ADC_Values_Q.SQL.Clear;
  ADC_Values_Q.SQL.Add (Format (SQL_READ_DATA_TXT, [M_DB_Start_id, M_DB_End_id]));
  ADC_Values_Q.Open;

  M_Data_Start_Index := M_Data_End_Index;
  While ADC_Values_Q.EOF = FALSE Do
    Begin { While }
      M_DB_id                      := ADC_Values_Q.Fields [0].AsLargeInt;
      M_Data_X_A[M_Data_End_Index] := ADC_Values_Q.Fields [1].AsDateTime;
      M_Data_Y_A[M_Data_End_Index] := ADC_Values_Q.Fields [2].AsInteger;
      ADC_Values_Q.Next;

      M_Data_End_Index := Validate_Array_Index (M_Data_End_Index + 1, DATA_ARRAY_SIZE);
    End; { While }
  ADC_Values_Q.Close;

  M_Data_A_Size := Validate_Array_Index (M_Data_End_Index - M_Data_Start_Index, DATA_ARRAY_SIZE);
  M_DB_Start_id := M_DB_End_id;
End; { TMain_F.Read_Data }


{#####################################################################################}
{ Chart                                                                               }
{#####################################################################################}

{ --------------------------------------------------------------------------------------- }
Procedure TMain_F.Chart_TTimer (Sender : TObject);
{ Repaint chart                                                                           }
{ --------------------------------------------------------------------------------------- }
Var
  I :                Integer;
  Index :            Integer;
  Time_Value :       TDateTime;
  Voltage :          Double;
  Old_Voltage :      Double;
  Zero_Crossings_C : Integer;
  Current_Extend :   TDoubleRect;
  Extend_Dif :       Double;

Begin { TMain_F.Chart_TTimer }
  { Read data from db }
  Read_Data;

  If M_Data_A_Size = 0 Then
    Begin { then }
      Exit;
    End; { then }

  Data_LS.BeginUpdate;

  Zero_Crossings_C := 0;
  Voltage          := 0;
  Old_Voltage      := Voltage;

  Index := M_Data_Start_Index;
  For I := 0 To M_Data_A_Size - 1 Do
    Begin { For }
      Time_Value := M_Data_X_A [Index];
      Voltage    := Calculate_Velocity (M_Data_Y_A [Index]);
      Data_LS.AddXY (Time_Value, Voltage);

      If ((Old_Voltage < 0) and (Voltage > 0) or (Old_Voltage > 0) and (Voltage < 0)) Then
        Begin { then }
          Inc (Zero_Crossings_C);
        End; { then }

      Old_Voltage := Voltage;
      Index       := Validate_Array_Index (Index + 1, DATA_ARRAY_SIZE);
    End; { For }

  Current_Extend             := Data_Chart_C.LogicalExtent;
  Extend_Dif                 := Current_Extend.b.X - Current_Extend.a.X;
  Current_Extend.b.X         := Time_Value;
  Current_Extend.a.X         := Current_Extend.b.X - Extend_Dif;
  Data_Chart_C.LogicalExtent := Current_Extend;

  Data_LS.EndUpdate;

  Output_Current_Data (M_Data_A_Size, Zero_Crossings_C, Time_Value);

  Show_Status ('');
End; { TMain_F.Chart_TTimer }


{ --------------------------------------------------------------------------------------- }
Procedure TMain_F.Chart_TS_DataPointHintToolHint (ATool : TDataPointHintTool; Const APoint : TPoint; Var AHint : String);
{ --------------------------------------------------------------------------------------- }
Const
  SPACE_START = '         ';

Var
  Time_Value : TDateTime;
  Voltage :    Double;

Begin { TMain_F.Chart_TS_DataPointHintToolHint }
    Try
      Time_Value := TChartSeries (ATool.Series).Source.Item [ATool.PointIndex]^.X;
      Voltage    := TChartSeries (ATool.Series).Source.Item [ATool.PointIndex]^.Y;
      AHint      := SPACE_START + 'X : ' + FormatDateTime ('hh:nn:ss.zzz', Time_Value) + #13 + Format (SPACE_START + 'Y : %.8f', [Voltage]);
    Except
      AHint := '';
    End; { Try }
End; { TMain_F.Chart_TS_DataPointHintToolHint }


End. { Main_Unit }
